/*
 * status.h --
 *
 *	Defines the statuses returned by Swarm functions. A status is
 *	defined in the following way:
 *
 *	MSB				LSB
 *	+-------+-------+-------+-------+
 *	|     Module	|    Message	|
 *	+-------+-------+-------+-------+
 *		16 bits		16 bits
 *
 *	Each module has its own set of statuses.
 *
 *	This scheme is derived from that used in the Sprite 
 *	operating system.
 *
 * Copyright 1999 Arizona Board of Regents
 *
 * $Header: /cvs/cvs/slinky/slinky/src2/libs/status/status.h,v 1.1 2004/01/16 20:00:07 jhh Exp $
 */

#ifndef _STATUS
#define _STATUS

/* 
 * This really should be int32, but including swarm.h leads to
 * circular dependencies.
 */
typedef unsigned int Status;

#define	STATUS_MODULE(status)		((status) >> 16)
#define STATUS_MSGNUM(status)		((status) & 0xFFFF)

/*
 * REMEMBER TO UPDATE STATUS.C
 */

/*
 * Status codes with module 0 are errnos. This makes it
 * backwards compatible.
 */


#define SUCCESS				0x00000000
#define FAILURE				0x10000000

/*
 * ELF. 
 */

#define ELF_HDR_BAD_MAGIC		0x00010000
#define ELF_HDR_BAD_CLASS		0x00010001
#define ELF_HDR_BAD_DATA		0x00010002
#define ELF_HDR_BAD_VERSION		0x00010003
#define ELF_HDR_BAD_SIZE		0x00010004
#define ELF_SHDR_BAD_SIZE		0x00010005
#define ELF_PHDR_BAD_SIZE		0x00010006
#define ELF_IO_ERROR			0x00010007
#define ELF_OUT_OF_BOUNDS		0x00010008

/* 
 * Status module itself
 */

#define STATUS_BADSTATUS		0x00020000

extern char 	*Status_Msg(Status status);
extern Status	Status_ToErrno(Status status, int *errnoValue);

#define StatusVerify(status, value) { \
    if ((status) != (value)) { \
	fprintf(stderr, "Unexpected status \"%s\" at %s:%d\n", \
	    Status_Msg(status), __FILE__, __LINE__); abort(); \
    }\
}

#endif /* _STATUS */

