#include <elf.h>
#include <stdio.h>
#include <stdlib.h>

#define assume(expr) \
    if (!(expr)) { \
    fprintf(stderr, "Assumption %s at %s:%d is false\n", __STRING(expr), \
    __FILE__, __LINE__); abort(); }

#ifdef XMALLOC
#include <publib.h>
#else
#define xmalloc(n) malloc(n)
#endif /* XMALLOC */

static char *cmdName = NULL;
static void 
Usage(void)
{
    fprintf(stderr, "Usage: %s executable\n", cmdName);
    exit(1);
}

static void
Debug(
    char *fmt,
    ...)
{
#ifdef DEBUG
    va_list	ap;

    va_start(ap, fmt);
    vfprintf(stderr, fmt, ap);
    va_end(ap);
#endif /* DEBUG */
}

int
main(int argc, char **argv)
{
    int		len;
    Elf32_Ehdr	ehdr;
    Elf32_Shdr	shdr;
    int		i,j;
    FILE	*f;
    int		n;
    Elf32_Dyn	*dyns;

    cmdName = argv[0];
    if (argc != 2) {
	Usage();
    }
    Debug("Opening %s\n", argv[1]);
    f = fopen(argv[1], "r+");
    assume(f != NULL);
    len = sizeof(Elf32_Ehdr);
    n = fread(&ehdr, len, 1, f);
    if (n != 1) {
	perror("Unable to read Elf header");
	exit(1);
    }
    assume(ehdr.e_ehsize == sizeof(Elf32_Ehdr));
    if ((ehdr.e_ident[EI_MAG0] != 0x7f) ||
	(ehdr.e_ident[EI_MAG1] != 'E') ||
	(ehdr.e_ident[EI_MAG2] != 'L') ||
	(ehdr.e_ident[EI_MAG3] != 'F')) {
	fprintf(stderr, "Bad magic in ELF hdr.\n");
	abort();
    }
    assume(ehdr.e_ident[EI_CLASS] == ELFCLASS32);
    assume(ehdr.e_ident[EI_DATA] == ELFDATA2LSB);
    assume(ehdr.e_ident[EI_VERSION] == EV_CURRENT);
    assume(ehdr.e_shentsize == sizeof(Elf32_Shdr));
    assume(ehdr.e_phentsize == sizeof(Elf32_Phdr));
    /*
     * Find the dynamic section.
     */
    n = fseek(f, ehdr.e_shoff, SEEK_SET);
    assume(n == 0);
    for (i = 0; i < ehdr.e_shnum; i++) {
	n = fread(&shdr, sizeof(shdr), 1, f);
	assume(n == 1);
	if (shdr.sh_type == SHT_DYNAMIC) {
	    Debug("Dynamic section is %d.\n", i);
	    assume(shdr.sh_size % sizeof(Elf32_Dyn) == 0);
	    dyns = xmalloc(shdr.sh_size);
	    n = fseek(f, shdr.sh_offset, SEEK_SET);
	    assume(n == 0);
	    n = fread(dyns, shdr.sh_size, 1, f);
	    assume(n == 1);
	    for (j = 0; j < shdr.sh_size / sizeof(Elf32_Dyn); j++) {
		if (dyns[j].d_tag == DT_RPATH) {
		    Debug("RPATH is %d\n", j);
		    dyns[j].d_un.d_val = 0;
		    break;
		}
	    }
	    n = fseek(f, shdr.sh_offset, SEEK_SET);
	    assume(n == 0);
	    n = fwrite(dyns, shdr.sh_size, 1, f);
	    assume(n == 1);
	    break;
	}
    }
    fclose(f);
    exit(0);
}
