#include <config.h>
#include <stdio.h>
#include <stdlib.h>
#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif
#ifdef HAVE_STRING_H
#include <string.h>
#endif
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif

#include <elflib.h>

#ifdef HAVE_LIBPUB
#include <publib.h>
#else
#define xmalloc(s) malloc(s)
#endif

#define VERSION "1.0"

#define assume(expr) \
    if (!(expr)) { \
    fprintf(stderr, "Assumption %s at %s:%d is false\n", __STRING(expr), \
    __FILE__, __LINE__); abort(); }

static char *cmdName = NULL;
static void 
Usage(void)
{
    fprintf(stderr, "Usage: %s [-V] input\n", cmdName);
    exit(1);
}

static void
Debug(
    char *fmt,
    ...)
{
#ifdef DEBUG
    va_list	ap;

    va_start(ap, fmt);
    vfprintf(stderr, fmt, ap);
    va_end(ap);
#endif /* DEBUG */
}

static Status
CheckSegment(
    Elf32_Phdr *phdrPtr,
    int		index)
{
    if (phdrPtr->p_align > 1) {
	if ((phdrPtr->p_vaddr % phdrPtr->p_align) != 
	    (phdrPtr->p_offset % phdrPtr->p_align)) {
	    printf("WARNING: segment %d is misaligned "
		    "(p_vaddr = 0x%x, p_offset = 0x%x, p_align = 0x%x)\n", 
		    index,
		    phdrPtr->p_vaddr, phdrPtr->p_offset, phdrPtr->p_align);
	}
    }
    return SUCCESS;
}

int
main(int argc, char **argv)
{
    Elf32_Ehdr	ehdr;
    Elf32_Shdr	*shdrs;
    Elf32_Phdr	*phdrs;
    int		i,j;
    FILE	*in;
    Status	status;
    int		option;

    cmdName = argv[0];
    do {
	option = getopt(argc, argv, "V");
	switch(option) {
	    case 'V':
		printf("%s Version %s\n", cmdName, VERSION);
		exit(0);
		break;
	    case -1:
		break;
	    case '?':
	    default:
		Usage();
	}
    } while (option != -1);
    if (argc - optind != 1) {
	Usage();
    }
    Debug("Opening %s\n", argv[optind]);
    in = fopen(argv[optind], "r");
    assume(in != NULL);
    status = Elf_EhdrRead(in, &ehdr);
    StatusVerify(status, SUCCESS);
    phdrs = xmalloc(ehdr.e_phnum * sizeof(Elf32_Phdr));
    status = Elf_PhdrReadAll(in, &ehdr, phdrs);
    for (i = 0; i < ehdr.e_phnum; i++) {
	status = CheckSegment(&phdrs[i], i);
	StatusVerify(status, SUCCESS);
    }
    shdrs = xmalloc(ehdr.e_shnum * sizeof(Elf32_Shdr));
    status = Elf_ShdrReadAll(in, &ehdr, shdrs);
    StatusVerify(status, SUCCESS);

    exit(0);
}

