#include <stdio.h>
#include <sys/ptrace.h>
#include <errno.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/user.h>

int
main(int argc, char **argv) {
    int		pid;
    int		status;
    int		rc;
    int		value;
    char	**args;
    struct user_regs_struct regs;
    int		i;

    if (argc < 2) {
	fprintf(stderr, "Usage: itrace cmd [args]\n");
	exit(1);
    }
    args = malloc(sizeof(char **) * argc);
    for (i = 1; i < argc; i++) {
	args[i-1] = argv[i];
    }
    args[i] = NULL;
    pid = fork();
    if (pid == 0) {
	rc = ptrace(PTRACE_TRACEME, 0, NULL, NULL);
	if (rc == -1) {
	    perror("PTRACE_TRACEME failed.\n");
	    exit(1);
	}
	rc = execv(args[0], args);
	if (rc == -1) {
	    perror("execl failed.\n");
	    exit(1);
	}
    } else {
	while(1) {
	    pid = wait(&status);
	    if(WIFSTOPPED(status)) {
		if (WSTOPSIG(status) != SIGTRAP) {
		    printf("Child stopped by signal %d.\n", WSTOPSIG(status));
		    exit(1);
		}
	    } else if (WIFSIGNALED(status)) {
		printf("Child killed by signal %d.\n", WTERMSIG(status));
	    } else {
		printf("Wait returned 0x%x\n", status);
	    }
	    errno = 0;
	    value = ptrace(PTRACE_GETREGS, pid, NULL, &regs);
	    if (errno) {
		perror("PTRACE_GETREGS failed\n");
		exit(1);
	    }
	    value = ptrace(PTRACE_PEEKTEXT, pid, regs.eip, NULL);
	    if (errno) {
		perror("PTRACE_PEEKTEXT failed\n");
		exit(1);
	    }
	    printf("pc = 0x%lx (0x%x)\n", regs.eip, value);
	    rc = ptrace(PTRACE_SINGLESTEP, pid, NULL, NULL);
	    if (rc == -1) {
		perror("PTRACE_CONT failed.\n");
		exit(1);
	    }
	}
    }
    exit(0);
}

