#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/mman.h>
#include <errno.h>

#ifdef HAVE_LIBPUB
#include <publib.h>
#else
#define xmalloc(s) malloc(s)
#endif

#include "slchunk.h"

static void Bin2Hex(u_char *hash, int len, char *string);
#define CHUNK_MIN	(1024 * 2)
#define CHUNK_MAX	(1024 * 64)

#define MAX_BOUNDARIES 100

static char *myName = NULL;

static void Usage(void);

static int compare(const void *a, const void *b) 
{
    return (* (int *) a) - (* (int *) b);
}

int
main(
    int	argc,
    char **argv)
{
    u_char		*base;
    u_char		*p;
    int			fd;
    struct stat		stat;
    u_char		*hash;
    char		string[HASHSTRLEN];
    int			rc;
    int			len;
    int			i;
    u_int64_t		fp;
    int			size;
    int			chunks;
    int			minChunk = CHUNK_MIN;
    int			maxChunk = CHUNK_MAX;
    int			option;
    int			boundary = 0;
    int			numBoundaries = 0;
    int			boundaries[MAX_BOUNDARIES];

    myName = argv[0];
    do {
	option = getopt(argc, argv, "m:M:b:");
	switch(option) {
	    case 'm':
		errno = 0;
		minChunk = strtol(optarg, NULL, 10);
		if (errno != 0) {
		    perror("Invalid minimum chunk size");
		    Usage();
		}
		break;
	    case 'M':
		errno = 0;
		maxChunk = strtol(optarg, NULL, 10);
		if (errno != 0) {
		    perror("Invalid maximum chunk size");
		    Usage();
		}
		break;
	    case 'b':
		if (numBoundaries >= MAX_BOUNDARIES) {
		    fprintf(stderr, "Too many boundaries.\n");
		    exit(1);
		} 
		errno = 0;
		boundaries[numBoundaries++] = strtol(optarg, NULL, 10);
		if (errno != 0) {
		    perror("Invalid boundary offset");
		    Usage();
		}
		break;
	    case -1:
		break;
	    case '?':
	    default:
		Usage();
	}
    } while(option != -1);
    if (optind == argc) {
	Usage();
    }
    if (minChunk >= maxChunk) {
	fprintf(stderr, "Min chunk must be < max chunk\n");
	exit(1);
    }
    if (numBoundaries > 1) {
	qsort(boundaries, numBoundaries, sizeof(int), compare);
    }
    fd = open(argv[optind], O_RDONLY);
    if (fd == -1) {
	perror("Unable to open input file");
	Usage();
    }
    rc = fstat(fd, &stat);
    assume(rc == 0);
    len = stat.st_size;
    base = mmap(0, len, PROT_READ, MAP_PRIVATE, fd, 0); 
    assume(base != MAP_FAILED);
    hash = xmalloc(HASHLEN);

#ifdef NOTDEF
    sha_buffer(base, len, (void *) hash);
    Bin2Hex(hash, HASHLEN, string);
    printf("SHA-1: %s\n", string);
    printf("\n");
    fp = fingerprint(base, len);
    printf("fp: 0x%llx\n", fp);
    printf("%4s %-40s %10s %5s\n", "", "SHA-1", "offset", "size");
#endif /* NOTDEF */

    chunks = 0;
    for (i = 0, size = 0; i < len; i += size) {
	int tmpMin; 
	int tmpMax;
	/*
	 * If we are at a boundary then advance to the next boundary.
	 */
	if ((boundary < numBoundaries) && (boundaries[boundary] == i)) {
	    boundary++;
	}
	if (boundary < numBoundaries) {
	    tmpMin = MIN(minChunk, boundaries[boundary] - i);
	    tmpMax = MIN(maxChunk, boundaries[boundary] - i);
	} else {
	    tmpMin = minChunk;
	    tmpMax = maxChunk;
	}
	//printf("i = %d, min = %d, max = %d\n", i, tmpMin, tmpMax);
	size = chunk(&base[i], len - i, tmpMin, tmpMax);
	sha_buffer(&base[i], size, (void *) hash);
	Bin2Hex(hash, HASHLEN, string);
	printf("%03d\t%s\t%10d\t%5d\n", chunks, string, i, size);
	chunks++;
    }
    exit(0);
}

void
Bin2Hex(
    u_char	*hash,
    int		len,
    char	*string)
{
    int		i;
    for (i = 0; i < len; i++) {
	sprintf(string, "%02x", hash[i]);
	string += 2;
    }
}

static void
Usage(void) 
{
    fprintf(stderr, "Usage: %s [-m minChunk] [-M maxChunk] file\n", myName);
    exit(1);
}

