#include <stdio.h>
#include <elflib.h>
#include <status.h>
#include <publib.h>
#include <stdlib.h>
#include <assert.h>
#include <errno.h>
#include <unistd.h>

int
main(int argc, char **argv) {
    Status	status;
    Elf_File	*filePtr;
    int		index;
    FILE	*in, *out;
    Elf_Segment	*segmentPtr;
    int		i;
    int		offset;
    int		n;
    int		len;

    if (argc != 4) {
	fprintf(stderr, "Usage: %s segment input output\n", argv[0]);
	exit(1);
    }
    errno = 0;
    index = strtol(argv[1], NULL, 10);
    if (errno) {
	perror("Invalid segment number");
	exit(1);
    }
    in = fopen(argv[2], "r");
    if (in == NULL) {
	perror("Unable to open input file");
	exit(1);
    }
    out = fopen(argv[3], "w");
    if (in == NULL) {
	perror("Unable to open output file");
	exit(1);
    }
    filePtr = Elf_FileNew();
    filePtr->name = argv[2];
    status = Elf_FileRead(in, filePtr);
    if (status != SUCCESS) {
	printf("Elf_FileRead failed: %s\n", Status_Msg(status));
	exit(1);
    }
    i = 0;
    LIST_FORALL(&filePtr->segmentList, (List_Links *) segmentPtr) {
	if (i == index) {
	    void	*segment;
	    offset = segmentPtr->phdr.p_offset;
	    n = fseek(in, offset, SEEK_SET);
	    len = segmentPtr->phdr.p_memsz;
	    segment = xmalloc(len);
	    fprintf(stderr, 
		"Dumping %d (0x%x) bytes starting at offset %d (0x%x)\n",
		    len, len, offset, offset);
	    n = fread(segment, len, 1, in);
	    if (n != 1) {
		perror("Unable to read segment");
		exit(1);
	    }
	    n = fwrite(segment, len, 1, out);
	    if (n != 1) {
		perror("Unable to write segment");
		exit(1);
	    }
	    exit(0);
	}
	i++;
    }
    fprintf(stderr, "Segment %d not found.\n", index);
    exit(1);
}




