#include <stdio.h>
#include <stdarg.h>
#include <slink.h>

#define VERSION "1.0"

static char *cmdName = NULL;
static void 
Usage(void)
{
    fprintf(stderr, "Usage: %s [-V] input [output]\n", cmdName);
    exit(1);
}

static void
Debug(
    char *fmt,
    ...)
{
#ifdef DEBUG
    va_list	ap;

    va_start(ap, fmt);
    vfprintf(stderr, fmt, ap);
    va_end(ap);
#endif /* DEBUG */
}

#define BUFSIZE (64 * 1024)

static void
CopyFile(
    FILE	*in,
    FILE	*out)
{
    int		size;
    int		n;
    static char	buffer[BUFSIZE];

    while (1) {
	size = fread(buffer, 1, BUFSIZE, in);
	assume (size >= 0);
	if (size == 0) {
	    break;
	}
	n = fwrite(buffer, 1, size, out);
	assume(n == size);
    };
    rewind(out);
}

int
main(int argc, char **argv)
{
    Elf32_Ehdr	ehdr;
    Elf32_Shdr	*shdrs;
    int		i,j;
    FILE	*in;
    FILE	*out = stdout;
    FILE	*tmp;
    Elf32_Dyn	*dyns;
    Status	status;
    int		option;

    cmdName = argv[0];
    do {
	option = getopt(argc, argv, "V");
	switch(option) {
	    case 'V':
		printf("%s Version %s\n", cmdName, VERSION);
		exit(0);
		break;
	    case -1:
		break;
	    case '?':
	    default:
		Usage();
	}
    } while (option != -1);
    if ((argc - optind < 1) || (argc - optind > 2)) {
	Usage();
    }
    Debug("Opening %s\n", argv[optind]);
    in = fopen(argv[optind], "r");
    assume(in != NULL);
    tmp = tmpfile();
    CopyFile(in, tmp);
    fclose(in);
    if (argc - optind == 2) {
	out = fopen(argv[optind+1], "w");
	if (out == NULL) {
	    perror("Unable to open output file");
	    exit(1);
	}
    }
    status = Elf_EhdrRead(tmp, &ehdr);
    StatusVerify(status, SUCCESS);
    shdrs = xmalloc(ehdr.e_shnum * sizeof(Elf32_Shdr));
    status = Elf_ShdrReadAll(tmp, &ehdr, shdrs);
    StatusVerify(status, SUCCESS);

    /*
     * Find the dynamic section and set the RPATH element to NULL.
     */
    for (i = 0; i < ehdr.e_shnum; i++) {
	if (shdrs[i].sh_type == SHT_DYNAMIC) {
	    Debug("Dynamic section is %d.\n", i);
	    assume(shdrs[i].sh_size % sizeof(Elf32_Dyn) == 0);
	    status = Elf_SectionRead(tmp, &shdrs[i], (void **) &dyns);
	    for (j = 0; j < shdrs[i].sh_size / sizeof(Elf32_Dyn); j++) {
		if (dyns[j].d_tag == DT_RPATH) {
		    Debug("RPATH is %d\n", j);
		    dyns[j].d_un.d_val = 0;
		    break;
		}
	    }
	    status = Elf_SectionWrite(tmp, &shdrs[i], (void *) dyns);
	    StatusVerify(status, SUCCESS);
	    break;
	}
    }
    /*
     * Find the .interp section and make the path relative.
     */
    for (i = 0; i < ehdr.e_shnum; i++) {
	char	*name;
	char	*path;
	char	*p;

	status = Elf_SectionGetName(tmp, &ehdr, &shdrs[i], &name); 
	StatusVerify(status, SUCCESS);
	if (strcmp(name, ".interp") == 0) {
	    status = Elf_SectionRead(tmp, &shdrs[i], (void **) &path);
	    StatusVerify(status, SUCCESS);
	    p = strrchr(path, '/');
	    if (p == NULL) {
		fprintf(stderr, "Unable to find / in path \"%s\"\n", path);
	    } else {
		memset(name, 0, shdrs[i].sh_size);
		strcpy(name, ".");
		strcat(name, p);
		status = Elf_SectionWrite(tmp, &shdrs[i], (void *) name);
		StatusVerify(status, SUCCESS);
	    }
	    break;
	}
    }
    rewind(tmp);
    CopyFile(tmp, out);
    exit(0);
}
