#ifndef _ELFLIB_H
#define _ELFLIB_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <elf.h>
#include <list.h>
#include <status.h>

typedef struct Elf_Segment {
    List_Links		links;
    Elf32_Phdr		phdr;
    struct Elf_Section	*sectionPtr;	// 1st section in segment
    char		*filename;
} Elf_Segment;

typedef struct Elf_SectionID {
    int		file;
    int		index;
} Elf_SectionID;

typedef struct Elf_Section {
    List_Links		links;
    char		*name;
    Elf32_Shdr		shdr;
    void		*section;
    Elf_Segment		*segmentPtr; // Segment if section is first in
				     // a segment.
    int			index;	
    Elf_SectionID	id;
    int			size;
    char		*filename;
} Elf_Section;


typedef struct Elf_File {
    List_Links		links;
    Elf32_Ehdr		ehdr;
    int			numSections;
    List_Links		sectionList;
    Elf_Section		*symbolTablePtr;
    Elf_Section		*stringPtr;
    int			numSegments;
    List_Links		segmentList;
    int			hdrSize;
    char		*name;
    int			id;
} Elf_File;

extern Status Elf_EhdrRead(FILE *, Elf32_Ehdr *);
extern Status Elf_PhdrRead(FILE *, Elf32_Ehdr *, int, Elf32_Phdr *);
extern Status Elf_PhdrReadAll(FILE *, Elf32_Ehdr *, Elf32_Phdr *);
extern Status Elf_ShdrRead(FILE *, Elf32_Ehdr *, int, Elf32_Shdr *);
extern Status Elf_ShdrReadAll(FILE *, Elf32_Ehdr *, Elf32_Shdr *);
extern Status Elf_SectionRead(FILE *, Elf32_Shdr *, void **);

extern Status Elf_SectionWrite(FILE *, Elf32_Shdr *, void *);

extern Status Elf_SectionGetName(FILE *, Elf32_Ehdr *, Elf32_Shdr *, char **);
extern Status Elf_FileRead(FILE *, Elf_File *);
extern Status Elf_GetSectionByIndex(Elf_File *, int, Elf_Section **);
extern Status Elf_GetSectionByName(Elf_File *, char *, Elf_Section **);
extern Status Elf_GetSymbol(Elf_File *, char *, Elf32_Sym **);
extern Status Elf_FileWrite(FILE *, Elf_File *);
extern Status Elf_FileDump(FILE *, Elf_File *);
extern Status Elf_GetSectionByID(Elf_File *filePtr, int file, int index, 
		    Elf_Section **sectionPtrPtr);
extern Elf_File		*Elf_FileNew(void);
extern Elf_Section	*Elf_SectionNew(void);
extern Elf_Segment	*Elf_SegmentNew(void);

#endif
