/* 
 * List_Sort.c --
 *
 *	Sorts a keyed list.
 *
 * @COPYRIGHT@
 */

#ifndef lint
static char rcsid[] __attribute__ ((unused)) = 
"$Header: /cvs/cvs/slinky/slinky/src2/libs/list/List_Sort.c,v 1.1 2004/01/16 20:00:07 jhh Exp $";
#endif /* not lint */

#include "list.h"


/*
 *----------------------------------------------------------------------
 *
 * List_Sort --
 *
 *	Sorts a keyed list.
 *
 * Results:
 *	Swarm return status.
 *
 * Side effects:
 *	The given list is sorted.
 *
 *----------------------------------------------------------------------
 */

Status
List_Sort(headerPtr, sortType)
    List_KeyedLinks	*headerPtr;	/* List to be sorted. */
    List_SortType	sortType;	/* Type of sort to be done. */
{
    List_KeyedLinks	*itemPtr;
    List_Links		*nextPtr;
    List_KeyedLinks	*prevPtr;
    int			insert;
    Status		status;

    LIST_FORALL_DEL((List_Links *) headerPtr, (List_Links *) itemPtr,
	    nextPtr) {
	LIST_FORALL((List_Links *) headerPtr, (List_Links *) prevPtr) {
	    if (prevPtr == itemPtr) {
		break;
	    }
	    insert = 0;
	    switch(sortType) {
		case LIST_SORT_ASCENDING:
		    switch(headerPtr->type) {
			case LIST_INT_KEY:
			    insert = (prevPtr->key <= itemPtr->key) ? 
					0 : 1;
			    break;
			case LIST_STRING_KEY:
			    insert = (strcmp(prevPtr->key, 
					itemPtr->key) <= 0) ? 0 : 1;
			    break;
			case LIST_OPAQUE_KEY:
			    status = FAILURE;
			    goto done;
		    }
		    break;
		case LIST_SORT_DESCENDING:
		    switch(headerPtr->type) {
			case LIST_INT_KEY:
			    insert = (prevPtr->key >= itemPtr->key) ? 
					0 : 1;
			    break;
			case LIST_STRING_KEY:
			    insert = (strcmp(prevPtr->key, 
					itemPtr->key) >= 0) ? 0 : 1;
			    break;
			case LIST_OPAQUE_KEY:
			    status = FAILURE;
			    goto done;
		    }
		    break;
		case LIST_OPAQUE_KEY:
		    status = FAILURE;
		    goto done;
	    }
	    if (insert) {
		break;
	    }
	}
	List_Move((List_Links *) itemPtr,
	    LIST_BEFORE((List_Links *) prevPtr));
    }
    status = SUCCESS;
done:
    return status;
}

/*
 *----------------------------------------------------------------------
 *
 * List_OpaqueSort --
 *
 *	Sorts a keyed list with opaque keys.
 *
 * Results:
 *	None.
 *
 * Side effects:
 *	The given list is sorted.
 *
 *----------------------------------------------------------------------
 */

void
List_OpaqueSort(headerPtr, sortType, compareFunc)
    List_KeyedLinks	*headerPtr;	/* List to be sorted. */
    List_SortType	sortType;	/* Type of sort to be done. */
    List_CompareElements compareFunc;
{
    List_KeyedLinks	*itemPtr;
    List_Links		*nextPtr;
    List_KeyedLinks	*prevPtr;
    int			insert;
    int			compare;

    LIST_FORALL_DEL((List_Links *) headerPtr, (List_Links *) itemPtr,
	    nextPtr) {
	LIST_FORALL((List_Links *) headerPtr, (List_Links *) prevPtr) {
	    if (prevPtr == itemPtr) {
		break;
	    }
	    insert = 0;
	    compare = (*compareFunc)(prevPtr->key, itemPtr->key);
	    switch(sortType) {
		case LIST_SORT_ASCENDING:
		    insert = (compare <= 0) ? 0 : 1;
		    break;
		case LIST_SORT_DESCENDING:
		    insert = (compare >= 0) ? 0 : 1;
		    break;
	    }
	    if (insert) {
		break;
	    }
	}
	List_Move((List_Links *) itemPtr,
	    LIST_BEFORE((List_Links *) prevPtr));
    }
}

